/*
 * WHICH.C
 *
 * Small minicrt wrapper app that searches the path or other environment
 * variable looking for the first matching file.
 *
 * Copyright (c) 2014 Malcolm J. Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdlib.h>
#include <tchar.h>
#include <minicrt.h>

LPTSTR searchfor;
LPTSTR searchvar = _T("PATH");

VOID
WhichUsage(LPTSTR strAppName)
{
    _tprintf(_T("Usage:\n")
             _T(" %s [-p var] file\n")
             _T(" If not specified, var defaults to PATH\n")
             _T(" If file extension not specified and var not specified, searches for files\n")
             _T("  ending in .COM, .EXE, .BAT and .CMD\n"),
             strAppName);

}

BOOL
WhichParseArgs(int argc, TCHAR* argv[])
{
    int i;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] == '/' || argv[i][0] == '-') {
            BOOL parsed = FALSE;
            if (_tcsicmp(&argv[i][1], _T("p")) == 0 &&
                argv[i + 1] != NULL) {

                i++;
                searchvar = argv[i];
                parsed = TRUE;
            }

            if (!parsed) {
                return FALSE;
            }
        } else {
            searchfor = argv[i];
        }
    }

    if (!searchfor) {
        return FALSE;
    }

    return TRUE;
}

#ifdef UNICODE
#define ENTRYPOINT wmain
#else
#define ENTRYPOINT main
#endif

int __cdecl
ENTRYPOINT (
    int argc,
    TCHAR* argv[]
    )
{
    TCHAR foundpath[MAX_PATH];
    int component = 0;
    BOOL exesearch = TRUE;
    int i;

    if (!WhichParseArgs(argc, argv)) {
        WhichUsage(argv[0]);
        return EXIT_FAILURE;
    }

    //
    //  If there's no extension, search for executable extensions
    //

    if (_tcschr(searchfor, '.') != NULL) {
        exesearch = FALSE;
    }

    if (exesearch) {
        TCHAR exefile[MAX_PATH];
        const TCHAR * exts[] = {_T(".com"),_T(".exe"),_T(".bat"),_T(".cmd")};
        int lowest_ext = -1;
        int lowest_ext_component;

        //
        //  Look for files with each extension in the search path.  Pick the first
        //  one in the path, and if two occur in the same path, pick the first
        //  one in the extension search order
        //

        for (i = 0; i < sizeof(exts)/sizeof(exts[0]); i++) {

            _stprintf_s(exefile, sizeof(exefile)/sizeof(exefile[0]), _T("%s%s"), searchfor, exts[i]);
            _tsearchenvex_s(exefile, searchvar, foundpath, sizeof(foundpath)/sizeof(foundpath[0]), &component);

            if (foundpath[0] != '\0' &&
                (lowest_ext == -1 || component < lowest_ext_component)) {

                lowest_ext_component = component;
                lowest_ext = i;
            }
        }

        //
        //  Now that we've picked an extension, go look it up again so we can
        //  display the result
        //

        if (lowest_ext != -1) {
            _stprintf_s(exefile, sizeof(exefile)/sizeof(exefile[0]), _T("%s%s"), searchfor, exts[lowest_ext]);
            _tsearchenvex_s(exefile, searchvar, foundpath, sizeof(foundpath)/sizeof(foundpath[0]), &component);
        }

    } else {

        //
        //  If we have a fully specified extension, just look for it directly
        //

        _tsearchenvex_s(searchfor, searchvar, foundpath, sizeof(foundpath)/sizeof(foundpath[0]), &component);
    }

    //
    //  Tell the user what we found, if anything
    //

    if (*foundpath) {
        _tprintf(_T("%s\n"), foundpath);
        return EXIT_SUCCESS;
    } else {
        _tprintf(_T("Not found\n"));
        return EXIT_FAILURE;
    }
}

// vim:sw=4:ts=4:et:
